﻿using System.Linq;
using System.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Diagrams.Core;

namespace WpfApp2
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            var toolService = this.diagram.ServiceLocator.GetService<IToolService>() as ToolService;
            toolService.ToolList[0] = new CustomPointerTool(this.diagram);            
        }
    }

    public class CustomPointerTool : PointerTool
    {
        private GroupService groupService;
        private RadDiagram graph;
        public CustomPointerTool(RadDiagram graph)
        {
            this.graph = graph;
            this.groupService = this.graph.ServiceLocator.GetService<IGroupService>() as GroupService;
        }

        public override bool MouseUp(PointerArgs e)
        {
            foreach (var shape in this.graph.Shapes.OfType<RadDiagramShape>())
            {
                shape.Tag = null;
            }
            return base.MouseUp(e);
        }


        public override bool MouseMove(PointerArgs e)
        {
            var baseResult = base.MouseMove(e);
            var selectionMode = this.graph.SelectionMode;
            if (this.HitItem == null && this.ToolService.IsMouseDown && selectionMode != SelectionMode.None)
            {
                foreach (var shape in this.graph.Shapes)
                {
                    var inflatedBounds = shape.ActualBounds.InflateRect(30);
                    if (inflatedBounds.Contains(e.TransformedPoint))
                    {
                        if (!shape.IsSelected)
                        {
                            bool canSelect = true;

                            var diagramShape = shape as RadDiagramShape;
                            if (diagramShape != null && diagramShape.Background == null)
                            {
                                
                                canSelect = IsPointOnVerticalLine(shape.ActualBounds.TopLeft, shape.ActualBounds.Height, e.TransformedPoint, 10) ||
                                            IsPointOnVerticalLine(shape.ActualBounds.TopRight, shape.ActualBounds.Height, e.TransformedPoint, 10) ||
                                            IsPointOnHorizontalLine(shape.ActualBounds.TopLeft, shape.ActualBounds.Width, e.TransformedPoint, 10) ||
                                            IsPointOnHorizontalLine(shape.ActualBounds.BottomLeft, shape.ActualBounds.Width, e.TransformedPoint, 10);

                                if (!canSelect)
                                {
                                    diagramShape.Tag = "DoNotSelect";
                                }
                            }

                            if (canSelect && (diagramShape.Tag == null || !diagramShape.Tag.Equals("DoNotSelect")))
                            {
                                bool addToExistingSelection = selectionMode != SelectionMode.Single && (this.ToolService.IsControlDown || selectionMode == SelectionMode.Multiple);
                                SelectWithGroups(addToExistingSelection, shape);
                            }

                        }
                        bool isDraggingEnabled = this.SelectionService.SelectedItems.Any(d => d.IsDraggingEnabled == false) ? false : true;
                        if (isDraggingEnabled)
                        {
                            this.ToolService.ActivateTool(DraggingTool.ToolName);
                            return false;
                        }

                        return true;
                    }
                }
            }
            return baseResult;
        }

        private bool IsPointOnVerticalLine(Point lineStart, double lineLength, Point searchPoint, double tolerance = 0)
        {
            double halfTolerance = tolerance / 2;
            var left = lineStart.X - halfTolerance;
            var top = lineStart.Y;

            var rect = new Rect(left, top, tolerance, lineLength);
            return rect.Contains(searchPoint);
        }

        private bool IsPointOnHorizontalLine(Point lineStart, double lineLength, Point searchPoint, double tolerance = 0)
        {
            double halfTolerance = tolerance / 2;
            var top = lineStart.Y - halfTolerance;
            var left = lineStart.X;

            var rect = new Rect(left, top, lineLength, tolerance);
            return rect.Contains(searchPoint);
        }

        private void SelectWithGroups(bool addToExistingSelection, IDiagramItem hitItem)
        {
            var itemsToSelect = this.groupService.GetItemSiblings(hitItem, addToExistingSelection).Union(new[] { hitItem });
            this.SelectionService.SelectItems(itemsToSelect, addToExistingSelection);
        }
    }
}
